//@ts-check
"use strict";

/**
@type {{
  pixiRoot: PIXI.Container;
  loop: PIXI.AnimatedSprite;
  logo: PIXI.Sprite;

  buttons: { text: PIXI.Text; action: Function; }[],
  activeButtonIndex: number;
  hoveredButtonIndex: number;

  logoTimer: number;
  loopTimer: number;
  loopForward: boolean;

  creditsTimer: number;
  creditsPages: PIXI.Container[];
}}
*/
let mainMenu = null;

const creditsEntryDuration = 4;

function registerMainMenuResources() {
  app.loader.add([
    "mainMenu/loop.png",
    "mainMenu/loop.sprite.json",
    "mainMenu/logo.png",
  ]);
}

function enterMainMenu() {
  ensureMenuMusic("mainMenu");

  const pixiRoot = new PIXI.Container();
  app.stage.addChildAt(pixiRoot, 0);
  app.renderer.backgroundColor = 0xffe1915b;

  const loopSpritesheet = getSpritesheet("mainMenu/loop");
  const loop = new PIXI.AnimatedSprite(loopSpritesheet.animations["loop"]);
  loop.anchor.x = 0.5;
  loop.y = 192;
  loop.play();
  pixiRoot.addChild(loop);

  const logo = new PIXI.Sprite(app.loader.resources["mainMenu/logo.png"].texture);
  logo.anchor.x = 0.5;
  logo.position.set(game.width / 2, 16);
  pixiRoot.addChild(logo);

  const buttons = [];

  function makeButton(action) {
    const text = new PIXI.Text("", new PIXI.TextStyle({
      fontFamily: "Arvo", fontSize: 64,
      fill: "#ffddb2", strokeThickness: 12, stroke: "#513a43", lineJoin: "bevel"
    }));
    text.scale.set(1 / 3);
    text.anchor.set(0, 0);
    pixiRoot.addChild(text);

    text.interactive = true;
    buttons.push({ text, action });
  }

  makeButton(() => leaveMainMenu(enterIntro)); // Play
  makeButton(openCredits); // Credits
  makeButton(cycleLanguage); // Language

  mainMenu = {
    pixiRoot,
    loop, logo,
    buttons,
    activeButtonIndex: -1,
    hoveredButtonIndex: -1,

    logoTimer: 0,

    loopTimer: 0,
    loopForward: true,

    creditsPages: [],
    creditsTimer: -1,
  };

  refreshMainMenuTexts();
  setActiveMainMenuButton(0);

  app.ticker.add(animateMainMenu);
  document.addEventListener("keydown", onMainMenuKeyDown);
  document.addEventListener("pointerdown", onMainMenuPointerDown);
}

function leaveMainMenu(enter) {
  setGameCursor("default");

  document.removeEventListener("pointerdown", onMainMenuPointerDown);
  document.removeEventListener("keydown", onMainMenuKeyDown);
  app.ticker.remove(animateMainMenu);

  mainMenu.pixiRoot.destroy();
  app.renderer.backgroundColor = 0x22222222;
  mainMenu = null;

  enter();
}

function animateMainMenu() {
  const dt = PIXI.Ticker.shared.elapsedMS / 1000;

  mainMenu.logoTimer = (mainMenu.logoTimer + dt / 5) % 1;
  mainMenu.logo.y = 16 + Math.sin(mainMenu.logoTimer * Math.PI * 2) * 4;

  mainMenu.loopTimer += 0.1 * (mainMenu.loopForward ? dt : -dt);
  if (Math.abs(mainMenu.loopTimer) > 1) {
    mainMenu.loopTimer = Math.sign(mainMenu.loopTimer);
    mainMenu.loopForward = !mainMenu.loopForward;
  }

  mainMenu.loop.x = game.width / 2 + Math.sign(mainMenu.loopTimer) * easeOutQuad(Math.abs(mainMenu.loopTimer)) * 80;

  if (mainMenu.creditsTimer >= 0) {
    mainMenu.creditsTimer += dt;

    const textIndex = Math.floor(mainMenu.creditsTimer / creditsEntryDuration);

    if (textIndex >= mainMenu.creditsPages.length) {
      closeCredits();
    } else {
      const progress = (mainMenu.creditsTimer % creditsEntryDuration) / creditsEntryDuration;

      for (let i = 0; i < mainMenu.creditsPages.length; i++) {
        const container = mainMenu.creditsPages[i];
        container.visible = (i === textIndex);
        if (!container.visible) continue;

        if (progress < 1 / 5) container.alpha = progress * 5;
        else if (progress > 4 / 5) container.alpha = 1 - (progress * 5 - 4);
        else container.alpha = 1;
      }
    }
  }

  mainMenu.hoveredButtonIndex = -1;

  if (mainMenu.creditsTimer < 0) {
    // Buttons
    const spacing = 60;
    let size = (mainMenu.buttons.length - 1) * spacing;
    mainMenu.buttons.forEach(b => size += b.text.width);

    let offset = 0;
    for (let i = 0; i < mainMenu.buttons.length; i++) {
      const button = mainMenu.buttons[i];
      button.text.position.set(game.width / 2 - size / 2 + offset, game.height - 40);
      button.text.updateTransform();

      const pos = new PIXI.Point(game.cursor.pos.x * game.stageScale, game.cursor.pos.y * game.stageScale);
      if (button.text.containsPoint(pos)) mainMenu.hoveredButtonIndex = i;

      offset += button.text.width + spacing;
    }
  }

  setGameCursor(mainMenu.hoveredButtonIndex !== -1 ? "pointer" : "default");
  if (mainMenu.hoveredButtonIndex !== -1) setActiveMainMenuButton(mainMenu.hoveredButtonIndex);
}

function setActiveMainMenuButton(index) {
  if (mainMenu.activeButtonIndex === index) return;

  mainMenu.activeButtonIndex = clamp(index, 0, mainMenu.buttons.length - 1);
  getSound("menus.select").play();

  for (let i = 0; i < mainMenu.buttons.length; i++) {
    const isActive = i === mainMenu.activeButtonIndex;
    const button = mainMenu.buttons[i];
    button.text.alpha = isActive ? 1 : 0.7;
    button.text.style.fill = isActive ? "#ffddb2" : "#aaa";
  }
}

function openCredits() {
  setGameCursor("default");

  for (const button of mainMenu.buttons) button.text.visible = false;

  function makeCreditsPage(name) {
    const container = new PIXI.Container();
    container.visible = false;
    container.position.set(game.width / 2, game.height * 5 / 7);
    mainMenu.pixiRoot.addChild(container);

    const titleText = new PIXI.Text(getText("credits." + name + ".title", null, ""), new PIXI.TextStyle({
      fontFamily: "Arvo", fontSize: 48,
      fill: "#ffddb2", strokeThickness: 12, stroke: "#513a43", lineJoin: "bevel",
      align: "center"
    }));
    titleText.scale.set(1 / 3);
    titleText.anchor.set(0.5, 0);
    container.addChild(titleText);

    const namesText = new PIXI.Text(getText("credits." + name + ".names", null, ""), new PIXI.TextStyle({
      fontFamily: "Arvo", fontSize: 64,
      fill: "#ffddb2", strokeThickness: 12, stroke: "#513a43", lineJoin: "bevel",
      align: "center"
    }));
    namesText.scale.set(1 / 3);
    namesText.anchor.set(0.5, 0);
    container.addChild(namesText);

    const height = titleText.height + namesText.height;
    titleText.y = -height / 2;
    namesText.y = -height / 2 + titleText.height;

    mainMenu.creditsPages.push(container);
  }

  makeCreditsPage("made");
  makeCreditsPage("art");
  makeCreditsPage("music");
  makeCreditsPage("code");
  makeCreditsPage("thanks");

  mainMenu.creditsTimer = 0;
  ensureMenuMusic("credits");
}

function closeCredits() {
  for (const page of mainMenu.creditsPages) page.destroy();
  mainMenu.creditsPages.length = 0;

  ensureMenuMusic("mainMenu");
  mainMenu.creditsTimer = -1;

  for (const container of mainMenu.creditsPages) container.visible = false;
  for (const button of mainMenu.buttons) button.text.visible = true;
}

function refreshMainMenuTexts() {
  mainMenu.buttons[0].text.text = getText("mainMenu.play");
  mainMenu.buttons[1].text.text = getText("mainMenu.credits");
  mainMenu.buttons[2].text.text = getText("mainMenu.language", { language: game.language.toUpperCase() });
}

function cycleLanguage() {
  setTextLanguage(game.language === "en" ? "fr" : "en");
  refreshMainMenuTexts();
}

function onMainMenuKeyDown(event) {
  if (mainMenu.creditsTimer < 0) {
    switch (event.code) {
      case "ArrowLeft":
      case "KeyQ":
      case "KeyA":
        setActiveMainMenuButton(mainMenu.activeButtonIndex - 1);
        break;
      case "ArrowRight":
      case "KeyD":
        setActiveMainMenuButton(mainMenu.activeButtonIndex + 1);
        break;
      case "Enter": mainMenu.buttons[mainMenu.activeButtonIndex].action(); getSound("menus.validate").play(); break;
    }
  } else {
    if (event.code === "Escape") { closeCredits(); getSound("menus.validate").play(); };
    if (event.code === "Enter") mainMenu.creditsTimer = Math.ceil(mainMenu.creditsTimer / creditsEntryDuration) * creditsEntryDuration;
  }
}

function onMainMenuPointerDown(event) {
  if (event.button !== 0) return;

  if (mainMenu.creditsTimer < 0 && mainMenu.hoveredButtonIndex !== -1) {
    mainMenu.buttons[mainMenu.hoveredButtonIndex].action(); getSound("menus.validate").play();
  } else {
    mainMenu.creditsTimer = Math.ceil(mainMenu.creditsTimer / creditsEntryDuration) * creditsEntryDuration;
  }
}